//////////////////////////////////////////////////////////////////////////
//             Copyright 2020  All Rights Reserved                //
//////////////////////////////////////////////////////////////////////////
/*
 * =======================================================================
 * 系统名 : 太的环卫平台
 * 子系统名 : 审批子系统
 * Class名 : ApprovalCoreController
 * 概要 :
 * 改版履历 :
 * 版本 日期 作者名 备注
 * 1.0.0 2020-09-08 guoenzhou 初次作成
 * =======================================================================
 */
package com.usky.oa.controller.web;

import com.usky.common.core.bean.ApiResult;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.service.OaApprovalCoreService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @Description: 审批申请单
 * @author: guoenzhou
 * @date:   2020-09-08
 * @Version: V1.0
 */
@Api(tags="审批流转控制")
@RestController
@RequestMapping("/approvalCore")
public class OaApprovalCoreController {

    @Autowired
    private OaApprovalCoreService oaApprovalCoreService;

    /**
     * 页面点击同意或者不同意处理按钮
     * @param opinion
     * @param status
     * @param approvalId
     * @return
     */
    @ApiOperation(value="审批流转-审批处理按钮", notes="页面点击同意或者不同意处理按钮")
    @RequestMapping(value = "/doApproval",method = RequestMethod.POST)
    @WebApi
    public ResultWrapper<?> doApproval(@RequestBody ApprovalRecordRequestVo bean) {
        UserTokenModel userTokenModel = SessionCache.get();
        Integer customerId = userTokenModel.getRootGroupId();
        Integer userId = userTokenModel.getUserId();
        ResultWrapper<?> result = oaApprovalCoreService.doApproval(bean.getOpinion(),bean.getStatus(),bean.getApprovalId(),userId);
        return result;
    }

    /**
     * 批量审批
     * @param opinion
     * @param status
     * @param approvalId
     * @return
     */
    @ApiOperation(value="审批流转-批量审批", notes="审批流转-批量审批")
    @RequestMapping(value = "/doBatchApproval",method = RequestMethod.POST)
    @WebApi
    public ResultWrapper<?> doBatchApproval(@RequestBody DoBatchApprovalRequestVo doBatchApprovalRequestVo) {
        UserTokenModel userTokenModel = SessionCache.get();
        Integer userId = userTokenModel.getUserId();
        Integer[] approvalIds = doBatchApprovalRequestVo.getApprovalIds();
        for (Integer approvalId : approvalIds) {
            oaApprovalCoreService.doApproval("", "G", approvalId, userId);
        }
        return ResultWrapper.ok("提交成功");
    }

    /**
     * 用户提交单据，调用此方法进入审批  submitDocument
     */
    @ApiOperation(value="审批流转-用户提交单据审批", notes="用户提交单据，调用此方法进入审批")
    @RequestMapping(value = "/submitDocument",method = {RequestMethod.GET,RequestMethod.POST})
    public ApiResult<?> submitDocument(@ApiParam("申请单ID")  @RequestParam Integer docId,
                                       @ApiParam("流程ID")  @RequestParam  Integer flowId) {
        Long userId = SecurityUtils.getUserId();
        return oaApprovalCoreService.submitDocument(docId,flowId,userId);
    }


    /**
     * 审批历史
     * @param docId
     */
    @ApiOperation(value="审批流转-审批历史", notes="审批流转-审批历史")
    @RequestMapping(value = "/findApprovalRecord", method = RequestMethod.GET)
    public ResultWrapper<List<ApprovalRecordReponseVo>> findApprovalRecord(@ApiParam("申请单ID") @RequestParam  Integer docId,
                                                                           @ApiParam("流程ID")   @RequestParam  Integer flowId) {
        List<ApprovalRecordReponseVo> result =  oaApprovalCoreService.approvalHistory(docId,flowId);
        return ResultWrapper.ok(result);
    }

    /**
     * 单据审批进程图
     * @param  flowId
     */
    @ApiOperation(value="审批流转-单据审批进程", notes="查看某个流程的审批链")
    @RequestMapping(value = "/getApprovalNodes", method = RequestMethod.GET)
    public ResultWrapper<List<ApprovalNodeVo>> getApprovalNodes(@ApiParam("申请单ID")  @RequestParam Integer flowId,
                                                                @ApiParam("流程ID")   @RequestParam Integer docId) {
        List<ApprovalNodeVo> result =  oaApprovalCoreService.getApprovalNodes(flowId,docId);
        return ResultWrapper.ok(result);
    }



    /**
     * 待我审批列表
     * @param docNo
     * @param docType
     * @param deptId
     * @param status
     * @param pageNo
     * @param pageSize
     * @return
     */
    @ApiOperation(value="审批流转-待我审批列表", notes="按照ID查询我审批的列表")
    @RequestMapping(value = "/findOwnerApproval", method = RequestMethod.GET)
    @WebApi
    public ResultWrapper<PaginationWrapper<OwnerApprovalReponse>>  findOwnerApproval(@ApiParam("单据单号") @RequestParam(name = "docNo", required = false )  String docNo,
                                                                                     @ApiParam("单据类型") @RequestParam(name = "docType", required = false) String docType,
                                                                                     @ApiParam("部门ID") @RequestParam(name = "deptId", required = false) Integer deptId,
                                                                                     @ApiParam("审批状态") @RequestParam(name = "status", required = false) String status,
                                                                                     @ApiParam("当前页") @RequestParam(name="page", defaultValue="1") Integer pageNo,
                                                                                     @ApiParam("一页显示条数") @RequestParam(name="pageSize", defaultValue="10") Integer pageSize
                                                               ) {
        UserTokenModel userTokenModel = SessionCache.get();
        Integer customerId = userTokenModel.getRootGroupId();
        Integer approvalUid = userTokenModel.getUserId();
        OwnerApprovalRequest ownerApprovalRequest = new OwnerApprovalRequest();
        if(StringUtils.isNotEmpty(docNo))ownerApprovalRequest.setDocNo(docNo);
        if(StringUtils.isNotEmpty(docType))ownerApprovalRequest.setDocType(docType);
        if(StringUtils.isNotEmpty(status))ownerApprovalRequest.setStatus(status);
        ownerApprovalRequest.setDeptId(deptId);
        ownerApprovalRequest.setPageNo(pageNo);
        ownerApprovalRequest.setPageSize(pageSize);
        PaginationWrapper<OwnerApprovalReponse> result = oaApprovalCoreService.findOwnerApproval(ownerApprovalRequest,approvalUid);
        return ResultWrapper.ok(result);
    }

    /**
     * 撤销申请
     * @param  flowId
     */
    @ApiOperation(value="审批流转-撤销申请", notes="审批流转-撤销申请")
    @RequestMapping(value = "/resetDoc", method = RequestMethod.GET)
    @WebApi
    public ResultWrapper<?> reset(@ApiParam("申请单ID")   @RequestParam Integer docId) {
        UserTokenModel userTokenModel = SessionCache.get();
        String userName = userTokenModel.getUserName();
        boolean bln  = oaApprovalCoreService.resetDoc(docId,userName);
        if(bln){
            return ResultWrapper.ok(bln);
        }
        return ResultWrapper.fail(4001,"撤销失败，请重试");

    }







}

