package com.usky.oa.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaApprovalFlow;
import com.usky.oa.service.OaApprovalFlowService;
import com.usky.oa.service.vo.OaApprovalFlowRequestVo;
import com.usky.oa.service.vo.OaApprovalNodeVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.stereotype.Controller;

import java.util.List;

/**
 * <p>
 * 审批_审批流程 前端控制器
 * </p>
 *
 * @author fu
 * @since 2024-09-30
 */
@Controller
@RequestMapping("/oaApprovalFlow")
public class OaApprovalFlowController {
    @Autowired
    private OaApprovalFlowService approvalFlowService;

    /**
     * 分页列表查询
     *
     * @param docType
     * @param pageNo
     * @param pageSize
     * @return
     */
    @ApiOperation(value="审批流程-分页列表查询", notes="审批流程-分页列表查询")
    @GetMapping("/list")
    public ApiResult<CommonPage<OaApprovalFlow>> queryPageList(
            @ApiParam("单据类型") @RequestParam(name="docType", required = false) String docType,
            @ApiParam("当前页") @RequestParam(name="page", defaultValue="1") Integer pageNo,
            @ApiParam("一页显示条数") @RequestParam(name="pageSize", defaultValue="10") Integer pageSize) {
        return ApiResult.success(approvalFlowService.page(docType,pageNo,pageSize));
    }

    /**
     * 添加
     *
     * @param approvalFlow
     * @return
     */
    @ApiOperation(value="审批流程-添加", notes="审批流程-添加")
    @PostMapping(value = "/add")
    public ApiResult<?> add(@RequestBody OaApprovalFlowRequestVo approvalFlow) {
        approvalFlowService.save(approvalFlow);
        return ApiResult.success("添加成功！");
    }

    /**
     * 编辑
     *
     * @param approvalFlow
     * @return
     */
    @ApiOperation(value="审批流程-编辑", notes="审批流程-编辑")
    @PutMapping(value = "/edit")
    public ApiResult<?> edit(@RequestBody OaApprovalFlowRequestVo approvalFlow) {
        approvalFlowService.edit(approvalFlow);
        return ApiResult.success("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value="审批流程-通过id删除", notes="审批流程-通过id删除")
    @DeleteMapping(value = "/delete")
    public ApiResult<?> delete(@RequestParam(name="id",required=true) Integer id) {
        approvalFlowService.removeById(id);
        return ApiResult.success("删除成功!");
    }


    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value="审批流程-通过id查询", notes="审批流程-通过id查询")
    @GetMapping(value = "/queryById")
    public ApiResult<?> queryById(@RequestParam(name="id",required=true) Integer id) {
        OaApprovalFlow approvalFlow = approvalFlowService.getById(id);
        return ApiResult.success(approvalFlow);
    }

    /**
     * 通过flowId查询
     *
     * @param flowId
     * @return
     */
    @ApiOperation(value="审批流程-通过id获取节点列表", notes="审批流程-通过id获取节点列表")
    @GetMapping(value = "/queryNodesByFlowId")
    public ApiResult<List<OaApprovalNodeVo>> queryNodesByFlowId(@RequestParam(name="flowId",required=true) Integer flowId) {
        List<OaApprovalNodeVo> result =  approvalFlowService.getFlowNodes(flowId);
        return ApiResult.success(result);
    }

    /**
     * 通过docType获取流程列表
     *
     * @param docType
     * @return
     */
    @ApiOperation(value="审批流程-通过docType获取流程列表", notes="审批流程-通过docType获取流程列表")
    @GetMapping(value = "/queryByDocType")
    @WebApi
    public ApiResult<List<ApprovalFlow>> queryByDocType(@ApiParam("单据类型") @RequestParam(name="docType",required=true) String docType) {
        UserTokenModel userTokenModel = SessionCache.get();
        Integer customerId = userTokenModel.getRootGroupId();
        List<ApprovalFlow> approvalFlow = approvalFlowService.queryByDocType(docType,customerId);
        return ApiResult.success(approvalFlow);
    }

    /**
     * 通过flowId获取审批中单据
     *
     * @param docType
     * @return
     */
    @ApiOperation(value="审批流程-通过flowId获取审批中单据", notes="审批流程-通过flowId获取审批中单据")
    @GetMapping(value = "/queryDocsByFlowId")
    public ApiResult<List<ApprovalDocument>> queryDocsByFlowId(@RequestParam(name="flowId",required=true) Integer flowId) {
        List<ApprovalDocument> approvalDocumentList = approvalFlowService.findDocsByFlowId(flowId);
        return ApiResult.success(approvalDocumentList);
    }
}

