package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaApprovalAccount;
import com.usky.oa.mapper.OaApprovalAccountMapper;
import com.usky.oa.mapper.OaApprovalFlowMapper;
import com.usky.oa.mapper.OaApprovalNodeMapper;
import com.usky.oa.service.OaApprovalAccountService;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 审批_账号权限 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-09-30
 */
@Service
public class OaApprovalAccountServiceImpl extends AbstractCrudService<OaApprovalAccountMapper, OaApprovalAccount> implements OaApprovalAccountService {
    @Autowired
    private OaApprovalAccountMapper approvalAccountMapper;

    @Autowired
    private OaApprovalNodeMapper approvalNodeMapper;

    @Autowired
    private OaApprovalFlowMapper approvalFlowMapper;

    @Autowired
    private GatewayInterfaceDao gatewayInterfaceDao;


    /**
     * 查询审批账号权限列表
     *
     * @param authType 权限类型
     * @param userId   用户id
     * @param docType  单据类型
     * @return
     */
    @Override
    public CommonPage<OaApprovalAccount> list(Integer authType, Integer userId, String docType, Integer pageNum, Integer pageSize) {
        Integer tenantId = SecurityUtils.getTenantId();
        LambdaQueryWrapper<OaApprovalAccount> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApprovalAccount::getTenantId, tenantId);
        queryWrapper.eq(authType != null, OaApprovalAccount::getAuthType, authType);
        queryWrapper.eq(userId != null, OaApprovalAccount::getUserId, userId);
        queryWrapper.eq(StringUtils.isNotBlank(docType), OaApprovalAccount::getDocType, docType);
        Page<OaApprovalAccount> oaApprovalAccountPage = approvalAccountMapper.selectPage(new Page<>(pageNum, pageSize), queryWrapper);
        List<OaApprovalAccount> records = oaApprovalAccountPage.getRecords();
        long total = oaApprovalAccountPage.getTotal();
        return new CommonPage<>(records, total, pageNum, pageSize);
    }

    @Override
    public boolean update(List<OaApprovalAccount> approvalAccountList, Integer customerId) {

        // 查询变更删除信息 TODO
       /* Integer userId = approvalAccountList.get(0).getUserId();
        ApprovalAccountExample approvalAccountExample = new ApprovalAccountExample();
        approvalAccountExample.createCriteria().andUserIdEqualTo(userId).andAuthTypeEqualTo(2);
        List<ApprovalAccount> resultList = approvalAccountMapper.selectByExample(approvalAccountExample);


        Map<Integer,ApprovalAccount> approvalAccountMap = approvalAccountList.stream().collect(Collectors.toMap(ApprovalAccount::getId, value -> value));

        resultList.stream().map(approvalAccount -> {
               if(!approvalAccountMap.containsKey(approvalAccount.getId())){
                   Integer userId = approvalAccount.getUserId();
                   String docType = approvalAccount.getDocType();
                   //查询该用户在该类型下是否有审批节点，如果有执行撤销机制
                   OaApprovalNodeExample approvalNodeExample = new OaApprovalNodeExample();
                   approvalNodeExample.createCriteria().andApprovalUidEqualTo(userId);
                   List<ApprovalNode> approvalNode = approvalNodeMapper.selectByExample(approvalNodeExample);
                   List<Integer> flowList = approvalNode.stream().map(ApprovalNode::getFlowId).distinct().collect(Collectors.toList());

                   ApprovalFlowExample approvalFlowExample = new ApprovalFlowExample();
                   approvalFlowExample.createCriteria().andIdIn(flowList);
                   List<ApprovalFlow> approvalFlows = approvalFlowMapper.selectByExample(approvalFlowExample);
                   List<ApprovalFlow> approvalFlows1 = approvalFlows.stream().filter(approvalFlow -> approvalFlow.getDocType().equals(docType)).collect(Collectors.toList());


               }

        });*/

//                Map<Integer, WorkShiftPeriodSnapshot> collect =
//                workShiftPeriodSnapshots.stream().collect(Collectors.toMap(WorkShiftPeriodSnapshot::getId, value -> value));


        // 操作，先删后加
        if (approvalAccountList.size() > 0) {
            Integer userId = approvalAccountList.get(0).getUserId();
            ApprovalAccountExample approvalAccountExample = new ApprovalAccountExample();
            approvalAccountExample.createCriteria().andUserIdEqualTo(userId);
            approvalAccountMapper.deleteByExample(approvalAccountExample);

            for (OaApprovalAccount approvalAccount : approvalAccountList) {
                approvalAccount.setTenantId(customerId);
                approvalAccountMapper.insertSelective(approvalAccount);
            }
            return true;
        }
        return false;
    }

    /**
     *
     * @param orgIds
     * @return
     */
    public List<Map<String, Integer>> findAvailableNum(List<Integer> orgIds) {
        List<Map<String, Integer>> resultList = new ArrayList<>();
        for (Integer orgId : orgIds) {
            Map<String, Integer> result = new HashMap<>();
            // 查询当前组织结构上限人数
            Integer limitNum = gatewayInterfaceDao.selectCountByEmployee(orgId);
            Integer availableNum = null;
            if (null != limitNum && limitNum > 0) {
                // 查询当前组织结构在线职员人数
                Integer currentNum = gatewayInterfaceDao.selectCurrentNumByEmployee(orgId);
                // 上限人数-在职人数=可用人数
                availableNum = limitNum - currentNum;
            }
            result.put("orgId", orgId);
            result.put("limitNum", limitNum);
            result.put("availableNum", availableNum);
            resultList.add(result);
        }
        return resultList;
    }

}
