package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.smartsanitation.common.pagination.PaginationWrapper;
import com.smartsanitation.common.response.ResultWrapper;
import com.tidecloud.sanitation.gateway.constant.OaApprovalConstant;
import com.tidecloud.sanitation.gateway.dao.ApprovalDocumentMapper;
import com.tidecloud.sanitation.gateway.dao.ApprovalNodeMapper;
import com.tidecloud.sanitation.gateway.dao.OaApprovalRecordMapper;
import com.tidecloud.sanitation.gateway.dao.ext.GatewayInterfaceDao;
import com.tidecloud.sanitation.gateway.model.entity.*;
import com.tidecloud.sanitation.gateway.service.ApprovalCoreService;
import com.tidecloud.sanitation.gateway.vo.*;
import com.usky.common.core.bean.ApiResult;
import com.usky.oa.constant.OaApprovalConstant;
import com.usky.oa.domain.OaApprovalDocument;
import com.usky.oa.domain.OaApprovalNode;
import com.usky.oa.domain.OaApprovalNodeExample;
import com.usky.oa.domain.OaApprovalRecord;
import com.usky.oa.domain.OaApprovalRecord;
import com.usky.oa.mapper.OaApprovalDocumentMapper;
import com.usky.oa.mapper.OaApprovalNodeMapper;
import com.usky.oa.mapper.OaApprovalRecordMapper;
import com.usky.oa.service.OaApprovalCoreService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;


/**
 *
 *
 * @description: 审批流转
 * @author: enzhouguo
 * @Date: 2020-09-08
 * @Version: V1.0
 */
@Service
public class OaApprovalCoreServiceImpl implements OaApprovalCoreService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ApprovalCoreServiceImpl.class);

    @Autowired
    private OaApprovalRecordMapper approvalRecordMapper;

    @Autowired
    private OaApprovalDocumentMapper approvalDocumentMapper;

    @Autowired
    private OaApprovalNodeMapper approvalNodeMapper;

    @Autowired
    private GatewayInterfaceDao gatewayInterfaceDao;


    /**
     * 单据审批
     * 页面点击同意或者不同意处理按钮
     * @param opinion
     * @param status
     * @param approvalId
     * @return
     */
    public ResultWrapper<?> doApproval(String opinion, String status, Integer approvalId, Integer userId) {
        // 修改审批记录
        try {
            LOGGER.info("userId: " + userId + "的" + approvalId + "审批操作开始");
            LambdaQueryWrapper<OaApprovalRecord> approvalRecordLambdaQueryWrapper = Wrappers.lambdaQuery();
            approvalRecordLambdaQueryWrapper.eq(OaApprovalRecord::getId,approvalId);
            OaApprovalRecord approvalRecord = approvalRecordMapper.selectOne(approvalRecordLambdaQueryWrapper);

            // approvalRecordService.getOaApprovalRecordById(approvalId)
            // 验证是否审批人审批
//            if(userId != approvalRecord.getApprovalUid()){
//                LOGGER.warn("该节点审批人ID:{},实际审批人：{}");
//                return ResultWrapper.fail(4001,"审批操作失败，你不能审批该节点 ");
//            }

            if (approvalRecord.getStatus().equals(OaApprovalConstant.APPROVAL_STATUS_F)) {
                approvalRecord.setOpinion(opinion);
                approvalRecord.setStatus(status);
                approvalRecord.setApprovalDate(LocalDateTime.now());
                approvalRecordMapper.updateById(approvalRecord);// 修改审批记录
                // 查询审批记录
                Integer nodeId = approvalRecord.getNodeId();
                if (status.equals(OaApprovalConstant.APPROVAL_STATUS_G)) {// 审批通过
                    LambdaQueryWrapper<OaApprovalNode> approvalNodeLambdaQueryWrapper = Wrappers.lambdaQuery();
                    approvalNodeLambdaQueryWrapper.eq(OaApprovalNode::getParentId, nodeId);
                    Integer nextNodeCount = approvalNodeMapper.selectCount(approvalNodeLambdaQueryWrapper);
                    if (nextNodeCount > 0) {// 有下
                        // 生成下一个审批节点
                        boolean bln = generateNextNode(approvalRecord.getDocId(), approvalRecord.getFlowId(), nodeId);
                    } else {
                        // 修改单据状态
                        OaApprovalDocument document = approvalDocumentMapper.selectByPrimaryKey(approvalRecord.getDocId());// approvalRecord.
                        document.setStatus(OaApprovalConstant.APPROVAL_STATUS_G);
                        approvalDocumentMapper.updateByPrimaryKey(document);
                    }

                } else if (status.equals(OaApprovalConstant.APPROVAL_STATUS_H)) {// 审批不通过
                    OaApprovalDocument document = approvalDocumentMapper.selectByPrimaryKey(approvalRecord.getDocId());// approvalRecord.
                    document.setStatus(OaApprovalConstant.APPROVAL_STATUS_H);
                    approvalDocumentMapper.updateByPrimaryKey(document);
                }
            } else {
                LOGGER.error("该审批节点已审批");
                return ResultWrapper.fail(4001, "审批操作失败，该审批节点已审批");
            }

        } catch (Exception exception) {
            LOGGER.error("审批操作失败", exception);
            return ResultWrapper.fail(4001, "审批操作失败，请重试");
        }
        return ResultWrapper.ok("操作成功");
    }

    /**
     * 生成下一个审批节点
     * @param
     * @return
     */
    private boolean generateNextNode(Integer docId, Integer flowId, Integer parentId) {
        // 根据doc_id查询单据内容
        LambdaQueryWrapper<OaApprovalDocument> documentLambdaQueryWrapper = Wrappers.lambdaQuery();
        documentLambdaQueryWrapper.eq(OaApprovalDocument::getId, docId);
        OaApprovalDocument document = approvalDocumentMapper.selectOne(documentLambdaQueryWrapper);
        // 根据流程ID寻找第一个审批节点
        LambdaQueryWrapper<OaApprovalNode> nodeLambdaQueryWrapper = Wrappers.lambdaQuery();
        nodeLambdaQueryWrapper.eq(OaApprovalNode::getFlowId, flowId)
                .eq(OaApprovalNode::getParentId, parentId);
        OaApprovalNode firstNode = approvalNodeMapper.selectList(nodeLambdaQueryWrapper).get(0);

        String isByDept = firstNode.getIsByDept();
        Integer approvalUid = null;
        if (StringUtils.isEmpty(isByDept) || "0".equals(isByDept)) {// 按指定人审批
            approvalUid = firstNode.getApprovalUserId();
        } else if ("1".equals(isByDept)) {// 按部门归属审批
            // TODO 后续考虑实现
        } else if ("2".equals(isByDept)) {// 按照部门职位审批
            // TODO 后续考虑实现
        }

        OaApprovalRecord approvalRecord = new OaApprovalRecord();
        approvalRecord.setFlowId(flowId);
        approvalRecord.setDocId(docId);
        approvalRecord.setNodeId(firstNode.getId());
        approvalRecord.setDocType(document.getDocType());
        approvalRecord.setStatus(OaApprovalConstant.APPROVAL_STATUS_F);// 审批中
        approvalRecord.setApprovalUserId(approvalUid);
        approvalRecord.setSubmitDate(document.getSubmitTime());
        approvalRecord.setTenantId(document.getCustomerId());
        boolean bln = approvalRecordMapper.insert(approvalRecord) > 0 ? true : false;
        return bln;
    }

    /**
     *
     * 单据提交
     * 调用此方法进入审批
     */
    public ApiResult<?> submitDocument(Integer docId, Integer flowId, Long userId) {
        // 验证单据是否本人提交
        LambdaQueryWrapper<OaApprovalDocument> documentLambdaQueryWrapper = Wrappers.lambdaQuery();
        documentLambdaQueryWrapper.eq(OaApprovalDocument::getId,docId);
        OaApprovalDocument document = approvalDocumentMapper.selectOne(documentLambdaQueryWrapper);
//        if(document.getProposer() != userId){
//            return ResultWrapper.fail(4001,"申请单只能申请人本人提交");
//        }

        // 生成第一个审批节点
        boolean bln = generateNextNode(docId, flowId, 0);
        if (bln) {
            // 修改单据状态为审批中
            document.setStatus(OaApprovalConstant.APPROVAL_STATUS_F);
            document.setUpdateTime(LocalDateTime.now());
            approvalDocumentMapper.updateById(document);
        }
        if (bln) {
            LOGGER.info("单据[{}]提交审批流成功", document.getDocNo());
            return ApiResult.success("单据[" + document.getDocNo() + "]提交审批流成功");
        } else {
            LOGGER.info("单据[{}]提交审批流失败", document.getDocNo());
            return ApiResult.error(String.valueOf(4001), "单据提交审批流失败");
        }
    }


    /**
     * 审批历史记录
     */
    public List<OaApprovalRecordReponseVo> approvalHistory(Integer docId, Integer flowId) {
//        OaApprovalNodeExample approvalNodeExample = new OaApprovalNodeExample();
//        approvalNodeExample.createCriteria().andFlowIdEqualTo(flowId);
//        List<ApprovalNode> approvalNodeList = approvalNodeMapper.selectByExample(approvalNodeExample);
//        Map<Integer,ApprovalNode> approvalNodeMap = new HashMap<>();
//        for (ApprovalNode approvalNode: approvalNodeList) {
//            approvalNodeMap.put(approvalNode.getId(),approvalNode);
//        }
//        OaApprovalRecordExample approvalRecordExample = new OaApprovalRecordExample();
//        approvalRecordExample.createCriteria().andDocIdEqualTo(docId);
//        List<OaApprovalRecord> result = approvalRecordMapper.selectByExample(approvalRecordExample);
//
//        List<OaApprovalRecordReponseVo> approvalRecordReponseVoList = new ArrayList<>();
//        for (OaApprovalRecord approvalRecord: result) {
//            ApprovalNode approvalNode = approvalNodeMap.get(approvalRecord.getNodeId());
//            OaApprovalRecordReponseVo approvalRecordReponseVo = new OaApprovalRecordReponseVo();
//            BeanUtils.copyProperties(approvalRecord,approvalRecordReponseVo);
//            approvalRecordReponseVo.setApprovalName("审批人"+approvalRecord.getApprovalUid());
//            approvalRecordReponseVo.setNodeName(approvalNode.getNodeName());
//            approvalRecordReponseVoList.add(approvalRecordReponseVo);
//        }
        List<OaApprovalRecordReponseVo> approvalRecordReponseVoList = gatewayInterfaceDao.selectOaApprovalRecord(docId);

        return approvalRecordReponseVoList;
    }


    /**
     * 单据审批链
     */
    public List<ApprovalNodeVo> getApprovalNodes(Integer flowId, Integer docId) {
//        OaApprovalRecordExample approvalRecordExample = new OaApprovalRecordExample();
//        approvalRecordExample.createCriteria().andDocIdEqualTo(docId);
//        List<OaApprovalRecord> result =approvalRecordMapper.selectByExample(approvalRecordExample);
//        Map<Integer,OaApprovalRecord> approvalRecordMap = new HashMap<>();
//        for (OaApprovalRecord approvalRecord: result) {
//            approvalRecordMap.put(approvalRecord.getNodeId(),approvalRecord);
//        }
//        OaApprovalNodeExample approvalNodeExample = new OaApprovalNodeExample();
//        approvalNodeExample.createCriteria().andFlowIdEqualTo(flowId);
//        List<ApprovalNode> approvalNodeList = approvalNodeMapper.selectByExample(approvalNodeExample);
//
//        List<ApprovalNodeVo> approvalNodeVoList = new ArrayList<>();
//        for (ApprovalNode approvalNode:approvalNodeList) {
//            ApprovalNodeVo approvalNodeVo = new ApprovalNodeVo();
//            BeanUtils.copyProperties(approvalNode,approvalNodeVo);
//            OaApprovalRecord approvalRecord = approvalRecordMap.get(approvalNode.getId());
//            if(null != approvalRecord) {
//                approvalNodeVo.setApprovalName("审批人" + approvalRecord.getApprovalUid());
//                approvalNodeVo.setStatus(approvalRecord.getStatus());
//                if (approvalNodeVo.getStatus().equals(OaApprovalConstant.APPROVAL_STATUS_F)) {
//                    approvalNodeVo.setIsApproval(1);
//                }
//            }
//            approvalNodeVoList.add(approvalNodeVo);
//        }
        List<ApprovalNodeVo> approvalNodeVoList = gatewayInterfaceDao.selectApprovalNodes(flowId, docId);

        return approvalNodeVoList;
    }


    /**
     * 我待审列表
     * @param ownerApprovalRequest
     * @param approvalUid
     * @return
     */
    public PaginationWrapper<OwnerApprovalReponse> findOwnerApproval(OwnerApprovalRequest ownerApprovalRequest, Integer approvalUid) {

        PaginationWrapper<OwnerApprovalReponse> paginationWrapper = new PaginationWrapper<>();
        Integer page = ownerApprovalRequest.getPageNo();
        Integer pageSize = ownerApprovalRequest.getPageSize();
        paginationWrapper.setPage(page);
        paginationWrapper.setPageSize(pageSize);

        // 通过uid查找我的审批记录
//        OaApprovalRecordExample approvalRecordExample = new OaApprovalRecordExample();
//        OaApprovalRecordExample.Criteria approvalRecordCriteria = approvalRecordExample.createCriteria();
//        if(null != approvalUid){
//            approvalRecordCriteria.andApprovalUidEqualTo(approvalUid);
//        }
//        if(null != ownerApprovalRequest.getStatus()) {
//            approvalRecordCriteria.andStatusEqualTo(ownerApprovalRequest.getStatus());
//        }
//        List<OaApprovalRecord> result  = approvalRecordMapper.selectByExample(approvalRecordExample);
//        List<Integer> docIds = result.stream().map(OaApprovalRecord::getDocId).collect(Collectors.toList());
//        Map<Integer,OaApprovalRecord> approvalRecordMap = result.stream().collect(Collectors.toMap(OaApprovalRecord::getDocId,value -> value));
//        for (OaApprovalRecord approvalRecord :result){
//            Integer docId = approvalRecord.getDocId();
//            docIds.add(docId);
//            approvalRecordMap.put(docId,approvalRecord);
//        }
        // 通过ApprovalUid
//        if(docIds.size() > 0) {
//            OaApprovalDocumentExample approvalDocumentExample = new OaApprovalDocumentExample();
//            OaApprovalDocumentExample.Criteria criteria = approvalDocumentExample.createCriteria();
//            approvalDocumentExample.setOrderByClause(" status,create_time desc");
//            criteria.andIdIn(docIds);
//            if (null != ownerApprovalRequest.getDocType()) {
//                criteria.andDocTypeEqualTo(ownerApprovalRequest.getDocType());
//            }
//            if (null != ownerApprovalRequest.getDocNo()) {
//                criteria.andDocNoEqualTo(ownerApprovalRequest.getDocNo());
//            }
//            if( null != ownerApprovalRequest.getDeptId() && ownerApprovalRequest.getDeptId() != 0) {
//                criteria.andDeptIdEqualTo(ownerApprovalRequest.getDeptId());
//            }
//            PageHelper.startPage(page, pageSize);
//            Page<OaApprovalDocument> approvalDocumentPage = (Page<OaApprovalDocument>)approvalDocumentMapper.selectByExample(approvalDocumentExample);
//            List<OwnerApprovalReponse>  ownerApprovalReponsesList = new ArrayList<>();
//            for (OaApprovalDocument document: approvalDocumentPage.getResult()) {
//                OaApprovalRecord approvalRecord = approvalRecordMap.get(document.getId());
//                OwnerApprovalReponse ownerApprovalReponse = new OwnerApprovalReponse();
//                BeanUtils.copyProperties(document, ownerApprovalReponse);
//                ownerApprovalReponse.setApprovalId(approvalRecord.getId());
//                ownerApprovalReponsesList.add(ownerApprovalReponse);
//            }
        PageHelper.startPage(page, pageSize);
        OwnerApprovalVo ownerApprovalVo = new OwnerApprovalVo();
        BeanUtils.copyProperties(ownerApprovalRequest, ownerApprovalVo);
        ownerApprovalVo.setApprovalUid(approvalUid);
        Page<OwnerApprovalReponse> approvalDocumentPage = (Page<OwnerApprovalReponse>) gatewayInterfaceDao.selectOwnerOaApprovalDocument(ownerApprovalVo);
        paginationWrapper.setTotalCount((int) approvalDocumentPage.getTotal());
        paginationWrapper.setData(approvalDocumentPage.getResult());

        return paginationWrapper;
//        }
//        return null;
    }

    /**
     * 撤销申请单
     * @param docId
     * @return
     */
    @Transactional
    public boolean resetDoc(Integer docId, String userName) {
        // 修改单据状态为审批中
        OaApprovalDocument document = approvalDocumentMapper.selectByPrimaryKey(docId);// approvalRecord.
        document.setStatus(OaApprovalConstant.APPROVAL_STATUS_I);
        approvalDocumentMapper.updateByPrimaryKeySelective(document);

        // 修改审批流程节点，审批中的记录都要删掉或置为撤销
        OaApprovalRecordExample approvalRecordExample = new OaApprovalRecordExample();
        approvalRecordExample.createCriteria().andDocIdEqualTo(docId).andStatusEqualTo("F");
        OaApprovalRecord approvalRecord = new OaApprovalRecord();
        approvalRecord.setStatus(OaApprovalConstant.APPROVAL_STATUS_I);
        approvalRecord.setOpinion("系统消息：" + userName + "手动撤销单据");
        approvalRecordMapper.updateByExampleSelective(approvalRecord, approvalRecordExample);
        return true;
    }

}
