package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.oa.domain.OaApprovalDocument;
import com.usky.oa.mapper.OaApprovalDocumentMapper;
import com.usky.oa.service.OaApprovalDocumentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 审批_申请单表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-09-30
 */
@Service
public class OaApprovalDocumentServiceImpl extends AbstractCrudService<OaApprovalDocumentMapper, OaApprovalDocument> implements OaApprovalDocumentService {

    @Autowired
    private OaApprovalDocumentMapper oaApprovalDocumentMapper;

    @Override
    public CommonPage<OaApprovalDocument> page(String docType, String status, String docNo, Integer pageNum, Integer pageSize) {
        // 创建分页对象
        Page<OaApprovalDocument> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<OaApprovalDocument> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(StringUtils.isNotBlank(docType), OaApprovalDocument::getDocType, docType)
                .eq(StringUtils.isNotBlank(status), OaApprovalDocument::getStatus, status)
                .eq(StringUtils.isNotBlank(docNo), OaApprovalDocument::getDocNo, docNo)
                .orderByDesc(OaApprovalDocument::getId);

        // 执行分页查询
        IPage<OaApprovalDocument> resultPage = oaApprovalDocumentMapper.selectPage(page, queryWrapper);

        // 如果查询结果为空，直接返回空的CommonPage对象
        if (resultPage.getRecords().isEmpty()) {
            return new CommonPage<>(Collections.emptyList(), 0, pageSize, pageNum);
        }

        // 创建返回的分页对象
        CommonPage<OaApprovalDocument> returnPage = new CommonPage<>(
                resultPage.getRecords(),
                resultPage.getTotal(),
                pageSize,
                pageNum
        );

        return returnPage;
    }

}
