package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.utils.bean.BeanUtils;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaApprovalDocument;
import com.usky.oa.domain.OaApprovalFlow;
import com.usky.oa.domain.OaApprovalNode;
import com.usky.oa.domain.OaApprovalRecord;
import com.usky.oa.mapper.*;
import com.usky.oa.service.OaApprovalAccountService;
import com.usky.oa.service.OaApprovalFlowService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.vo.OaApprovalFlowRequestVo;
import com.usky.oa.service.vo.OaApprovalNodeVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 审批_审批流程 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-09-30
 */
@Service
public class OaApprovalFlowServiceImpl extends AbstractCrudService<OaApprovalFlowMapper, OaApprovalFlow> implements OaApprovalFlowService {
    @Autowired
    private OaApprovalFlowMapper approvalFlowMapper;
    @Autowired
    private OaApprovalNodeMapper approvalNodeMapper;
    @Autowired
    private OaApprovalDocumentMapper approvalDocumentMapper;
    @Autowired
    private OaApprovalRecordMapper approvalRecordMapper;

    @Override
    public CommonPage<OaApprovalFlow> page(String docType, Integer pageNum, Integer pageSize) {
        LambdaQueryWrapper<OaApprovalFlow> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(StringUtils.isNotBlank(docType), OaApprovalFlow::getDocType, docType)
                .orderByDesc(OaApprovalFlow::getId);
        Page<OaApprovalFlow> oaApprovalFlowPage = approvalFlowMapper.selectPage(new Page<>(pageNum, pageSize), queryWrapper);
        List<OaApprovalFlow> records = oaApprovalFlowPage.getRecords();
        long total = oaApprovalFlowPage.getTotal();
        CommonPage<OaApprovalFlow> oaApprovalFlowCommonPage = new CommonPage<>(records, total, pageNum, pageSize);
        return oaApprovalFlowCommonPage;
    }

    @Override
    public Boolean save(OaApprovalFlowRequestVo flowRequestVo) {
        OaApprovalFlow approvalFlow = new OaApprovalFlow();
        BeanUtils.copyProperties(flowRequestVo,approvalFlow);
        approvalFlow.setCreateTime(LocalDateTime.now());
        approvalFlow.setTenantId(SecurityUtils.getTenantId());
        approvalFlowMapper.insert(approvalFlow);
        Integer flowId = approvalFlow.getId();

        List<OaApprovalNodeVo> approvalNodeVos = flowRequestVo.getNodes();
        if(approvalNodeVos!= null && !approvalNodeVos.isEmpty()){
            int parentId = 0;
            for (OaApprovalNodeVo approvalNodeVo : approvalNodeVos) {
                OaApprovalNode approvalNode = new OaApprovalNode();
                BeanUtils.copyProperties(approvalNodeVo,approvalNode);
                approvalNode.setCustomerId(flowRequestVo.getCustomerId());
                approvalNode.setFlowId(flowId);
                approvalNode.setParentId(parentId);
                approvalNodeMapper.insert(approvalNode);
                parentId = approvalNode.getId();
            }
        }
        return true;
    }

    @Override
    @Transactional
    public boolean edit(OaApprovalFlowRequestVo approvalFlowVo) {
        String username = SecurityUtils.getUsername();
        Integer tenantId = SecurityUtils.getTenantId();
        approvalFlowVo.setUpdateBy(username);
        approvalFlowVo.setCustomerId(tenantId);

        //可以分两种情况做，一种是流程已经被单据引用 逻辑删除创建新的版本  一种是流程未被引用可以
        Integer flowId = null;
        if(findCountByFlowId(approvalFlowVo.getId()) > 0) {
            //step1:逻辑删除流程和流程节点
            OaApprovalFlow approvalFlow = new OaApprovalFlow();
            approvalFlow.setId(approvalFlowVo.getId());
            approvalFlow.setUpdateBy(approvalFlowVo.getUpdateBy());
            approvalFlow.setIsDelete(1);
            approvalFlowMapper.updateById(approvalFlow);


            //step2:添加新版本的流程节点
            OaApprovalFlow approvalFlow1 = new OaApprovalFlow();
            BeanUtils.copyProperties(approvalFlowVo, approvalFlow1);
            approvalFlow1.setCreateBy(approvalFlowVo.getUpdateBy());
            approvalFlowMapper.insert(approvalFlow1);
            flowId = approvalFlow1.getId();

            List<OaApprovalNodeVo> approvalNodeVos = approvalFlowVo.getNodes();
            if (approvalNodeVos != null && approvalNodeVos.size() > 0) {
                int parentId = 0;
                for (OaApprovalNodeVo approvalNodeVo : approvalNodeVos) {
                    OaApprovalNode approvalNode = new OaApprovalNode();
                    BeanUtils.copyProperties(approvalNodeVo, approvalNode);
                    approvalNode.setCustomerId(approvalFlowVo.getCustomerId());
                    approvalNode.setFlowId(flowId);
                    approvalNode.setParentId(parentId);
                    approvalNode.setId(null);
                    approvalNodeMapper.insert(approvalNode);
                    parentId = approvalNode.getId();
                }
            }

            //step3:流程中单据变更并且更新流程ID //撤销所有流程中的单据
            /******begin 需求变更：流程变更重新重新生成单据********/
            LambdaQueryWrapper<OaApprovalDocument> criteria = Wrappers.lambdaQuery();
            criteria.eq(OaApprovalDocument::getCustomerId, approvalFlowVo.getId())
                            .in(OaApprovalDocument::getStatus, Arrays.asList("F","I"));
            OaApprovalDocument approvalDocument = new OaApprovalDocument();
            approvalDocument.setStatus("I");
            approvalDocument.setFlowId(flowId);
            approvalDocumentMapper.update(approvalDocument, criteria);
            /****end****/
//            OaApprovalDocumentExample approvalDocumentExample = new OaApprovalDocumentExample();
//            OaApprovalDocumentExample.Criteria criteria = approvalDocumentExample.createCriteria();
//            criteria.andFlowIdEqualTo(approvalFlowVo.getId());
//            criteria.andStatusIn(Arrays.asList(OaApprovalConstant.APPROVAL_STATUS_F,OaApprovalConstant.APPROVAL_STATUS_I));//审批流程，审批中和撤销单据都要调整为新流程ID
//            List<OaApprovalDocument> documentList = approvalDocumentMapper.selectByExample(approvalDocumentExample);
//            for (OaApprovalDocument approvalDocument:documentList) {
//                //先撤销旧单据
//                approvalDocument.setStatus(OaApprovalConstant.APPROVAL_STATUS_I);
//                approvalDocumentMapper.updateByPrimaryKeySelective(approvalDocument);
//                //再复制增加新单据
//                approvalDocument.setStatus(OaApprovalConstant.APPROVAL_STATUS_E);
//                approvalDocument.setFlowId(flowId);
//                approvalDocument.setId(null);
//                approvalDocument.setDocNo(approvalDocument.getDocType()+"-"+DateUtil.formate(new Date(),"yyMMddHHmmss"));
//                approvalDocumentMapper.insertSelective(approvalDocument);
//            }
            /********/

            //step4:修改审批流程节点，审批中的记录都要删掉或置为撤销
            LambdaQueryWrapper<OaApprovalRecord> recordQuery = Wrappers.lambdaQuery();
            recordQuery.eq(OaApprovalRecord::getFlowId,approvalFlowVo.getId()).eq(OaApprovalRecord::getStatus,"F");

            OaApprovalRecord approvalRecord = new OaApprovalRecord();
            approvalRecord.setStatus("I");
            approvalRecord.setOpinion("系统消息：审批节点调整，单据撤销");
            approvalRecordMapper.update(approvalRecord,recordQuery);



        } else {
            //流程未被用处理方法
            OaApprovalFlow approvalFlow1 = new OaApprovalFlow();
            BeanUtils.copyProperties(approvalFlowVo, approvalFlow1);
            approvalFlow1.setCreateUser(approvalFlowVo.getUpdateUser());
            approvalFlowMapper.updateByPrimaryKeySelective(approvalFlow1);

            //根据流程ID删除节点
            OaApprovalNodeExample approvalNodeExample = new OaApprovalNodeExample();
            approvalNodeExample.createCriteria().andFlowIdEqualTo(approvalFlowVo.getId());
            approvalNodeMapper.deleteByExample(approvalNodeExample);

            //重新添加节点
            List<ApprovalNodeVo> approvalNodeVos = approvalFlowVo.getNodes();
            if (approvalNodeVos != null && approvalNodeVos.size() > 0) {
                int parentId = 0;
                for (ApprovalNodeVo approvalNodeVo : approvalNodeVos) {
                    ApprovalNode approvalNode = new ApprovalNode();
                    BeanUtils.copyProperties(approvalNodeVo, approvalNode);
                    approvalNode.setCustomerId(approvalFlowVo.getCustomerId());
                    approvalNode.setFlowId(approvalFlowVo.getId());
                    approvalNode.setParentId(parentId);
                    approvalNode.setId(null);
                    approvalNodeMapper.insertSelective(approvalNode);
                    parentId = approvalNode.getId();
                }
            }
        }

        return true;
    }

    @Override
    public List<OaApprovalNodeVo> getFlowNodes(Integer flowId) {
        OaApprovalNodeExample approvalNodeExample = new OaApprovalNodeExample();
        approvalNodeExample.createCriteria().andFlowIdEqualTo(flowId);
        List<ApprovalNode> approvalNodeList = approvalNodeMapper.selectByExample(approvalNodeExample);

        List<ApprovalNodeVo> approvalNodeVoList = new ArrayList<>();
        for (ApprovalNode approvalNode:approvalNodeList) {
            ApprovalNodeVo approvalNodeVo = new ApprovalNodeVo();
            BeanUtils.copyProperties(approvalNode,approvalNodeVo);
            approvalNodeVoList.add(approvalNodeVo);
        }

        return approvalNodeVoList;
    }

    private  long findCountByFlowId(Integer flowId) {
        OaApprovalDocumentExample approvalDocumentExample = new OaApprovalDocumentExample();
        OaApprovalDocumentExample.Criteria criteria = approvalDocumentExample.createCriteria();
        criteria.andFlowIdEqualTo(flowId);
        return  approvalDocumentMapper.countByExample(approvalDocumentExample);
    }


}
